entity i2c_1 is
port ( CLK_66MHz : IN std_logic;                           --FPGA System clock 66MHz
        RESET_N   : IN std_logic;                          --Active low FPGA Reset 
        SCL       : OUT std_logic;                         --I2C Serial clock line
        SDA          : INOUT std_logic;                       --I2C Serial data line
        ACK       : OUT std_logic;                         --Slave Acknowledge bit
--      AddressIn : IN std_logic_vector (7 downto 0);      --Slave's internal register address
--      RegAddress: IN std_logic_vector (7 downto 0);       
--      DataIn    : IN std_logic_vector (7 downto 0);      --MSB data written to slave's internal register
--      DataIn_lower : IN std_logic_vector (7 downto 0);   --LSB data written to slave's internal register
        DATA1     : OUT std_logic_vector (7 downto 0);
        DATA2     : OUT std_logic_vector (7 downto 0);
        s_tate      : INOUT std_logic_vector (7 downto 0)
      );
end i2c_1;

architecture behav of i2c_1 is
signal SDA_IN : std_logic;                                                  --SDA input buffer
signal SDA_OE : std_logic;                                                    --output buffer enable, high=input (hi-Z output), low=active drive
signal SDA_OUT: std_logic;                                                  --SDA output buffer
signal bit_cnt: std_logic_vector (2 downto 0);                      --3-bit counter (0 to 7) (8 counts) 
signal I2C_clk_cnt : std_logic_vector (15 downto 0);            --Divided I2C clock count
signal CLK_I2C : std_logic;                                                    --Clock for I2C R/W operation
signal msb_data : std_logic_vector (7 downto 0);                --MSB data read from slave's internal register (Bit 15 to 8)
signal lsb_data : std_logic_vector (7 downto 0);                   --LSB data read from slave's internal register (Bit 7 to 0)

constant RegAddress : std_logic_vector (7 downto 0) := "00011111";      --Slave's internal register address
constant AddressIn  : std_logic_vector (7 downto 0) := "00000000";        --Slave's internal register address
constant DataIn     : std_logic_vector (7 downto 0) := "00100001";           --MSB data written to slave's internal register (Config reg1)
constant DataIn_lower : std_logic_vector (7 downto 0) := "10011111";        --LSB data written to slave's internal register (Config reg1)
constant SlaveAddress_Read  : std_logic_vector (7 downto 0) := "10111001";  --7-bit Slave address + read
constant SlaveAddress_Write : std_logic_vector (7 downto 0) := "10111000";  --7-bit Slave address + write
constant CLK_Freq : integer := 66000000;                                    --FPGA Clock frequency 66MHz
constant I2C_freq : integer := 100000;                                      --Clock for I2C operation 100kHz

signal s_cl : std_logic;                                                    --serial clock buffer
signal a_ck : std_logic;                                                    --slave ACK bufffer
signal datacheck : std_logic_vector (7 downto 0);                           --buffered data register

begin
SCL <= s_cl;    --buffered serial clock assigned to main I2C master's serial clock
ACK <= a_ck;    --buffered slave ACK assigned to output to monitor the ACK bit

----------------------------Clock generation for I2C operation----------------------------
process (CLK_66MHz, RESET_N)
 begin       
    if (RESET_N = '0') then
       I2C_clk_cnt <= (others => '0');
       CLK_I2C <= '0';
    elsif (rising_edge (CLK_66MHz)) then
       if (I2C_clk_cnt < conv_std_logic_vector ((CLK_Freq/I2C_freq), 16)) then   
           I2C_clk_cnt <= I2C_clk_cnt + "0000000000000001";
       else   
            I2C_clk_cnt <= (others => '0');
       end if;    

       if (I2C_clk_cnt < conv_std_logic_vector ((CLK_Freq/(I2C_freq*2)), 16)) then
            CLK_I2C <= '0';
       else   
            CLK_I2C <= '1';
       end if;
    end if;   
end process;

--i2cclk_chk <= CLK_I2C;
------------------------------------------------------------------------------------------

----------------------------Status of SDA line when input and output---------------------
SDA <= SDA_OUT when SDA_OE = '1' else 'Z';
SDA_IN <= SDA when SDA_OE = '0' else 'H';         
------------------------------------------------------------------------------------------

------------------------------I2C Read/Write frame formation------------------------------
process (CLK_I2C, RESET_N)
 begin
        if (RESET_N = '0') then                       --reset asserted and status of signals on POR
            s_cl <= '1';
            SDA_OE <= '1';
            SDA_OUT <= '1';
            a_ck <= '1';
            s_tate <= "00000000";
            bit_cnt <= "111";    

        elsif (rising_edge (CLK_I2C)) then         
            case (s_tate) is
               --Start--     
                when "00000000" =>  --8'h0        --Idle state                                 
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "00000000";
                    s_tate <= "00000001";                

                when "00000001" =>  --8'h01
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '0';
                    bit_cnt <= "111";
                    datacheck <= "00000001";
                    s_tate <= "00000010";

               --Slave Address + Write--   
                when "00000010" =>  --8'h02         
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= SlaveAddress_Write (conv_integer(bit_cnt));
                    datacheck <= "00000010";
                    s_tate <= "00000011";    

                when "00000011" =>  --8'h03
                    s_cl <= '1';
                    if (bit_cnt >= "001") then   
                        bit_cnt <= bit_cnt - "001";
                        s_tate <= "00000010";
                    else
                        bit_cnt <= "111";
                        datacheck <= "00000011";
                        s_tate <= "00000100";
                    end if;               

                --Slave ACK Bit--   
                when "00000100" =>  --8'h04
                    s_cl <= '0';
                    SDA_OE <= '0';
                    SDA_OUT <= '1';
                    datacheck <= "00000100";
                    s_tate <= "00000101";                    

                when "00000101" =>  --8'h05
                    s_cl <= '1';
                    SDA_OE <= '0';
                    a_ck <= SDA_IN;
                    datacheck <= SlaveAddress_Write;
                    if (SDA_IN = '1') then
                        datacheck <= "00010000";
                        s_tate <= "11101110";
                    else
                        datacheck <= "00000101";
                        s_tate <= "00010001";
                    end if;             

                --Internal Register Address--   
                when "00010001" =>  --8'h11
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= AddressIn (conv_integer(bit_cnt));
                    datacheck <= "00010001";
                    s_tate <= "00010010";                   

                when "00010010" =>  --8'h12
                    s_cl <= '1';
                    if (bit_cnt >= "001") then
                        bit_cnt <= bit_cnt - "001";
                        s_tate <= "00010001";
                    else
                        bit_cnt <= "111";
                        datacheck <= "00010010";
                        s_tate <= "00010011";
                    end if;

                --Slave ACK Bit--
                when "00010011" =>  --8'h13
                    s_cl <= '0';
                    SDA_OE <= '0';
                    SDA_OUT <= '1';
                    datacheck <= "00010011";
                    s_tate <= "00010100";

                when "00010100" =>  --8'h14
                    s_cl <= '1';
                    SDA_OE <= '0';
                    a_ck <= SDA_IN;
                    datacheck <= AddressIn;
                    if (SDA_IN = '1') then
                        datacheck <= "00100000";
                        s_tate <= "11101110";
                    else
                        datacheck <= "00010100";
                        s_tate <= "00100001";
                    end if;

                --Write MSB Data (Bit 15 to 8)   
                when "00100001" =>  --8'h21
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= DataIn (conv_integer (bit_cnt));
                    datacheck <= "00100001";
                    s_tate <= "00100010";

                when "00100010" =>  --8'h22
                    s_cl <= '1';
                    if (bit_cnt >= "001") then
                        bit_cnt <= bit_cnt - "001";
                        s_tate <= "00100001";
                    else
                        bit_cnt <= "111";
                        datacheck <= "00100010";
                        s_tate <= "00100011";
                    end if;

                --Slave ACK Bit--   
                when "00100011" =>  --8'h23
                    s_cl <= '0';
                    SDA_OE <= '0';
                    SDA_OUT <= '1';
                    datacheck <= DataIn;
                    s_tate <= "00100100";
                  
                when "00100100" =>  --8'h24
                    s_cl <= '1';
                    SDA_OE <= '0';
                    a_ck <= SDA_IN;
                    if (SDA_IN = '1') then
                        datacheck <= "01010000";
                        s_tate <= "11101110";
                    else
                        datacheck <= "00100100";
                        s_tate <= "00100101";
                    end if;

                --Write LSB Data (Bit 7 to 0)   
                when "00100101" =>  --8'h25
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= DataIn_lower(conv_integer(bit_cnt));
                    datacheck <= "00100101";
                    s_tate <= "00100110";

                when "00100110" =>  --8'h26
                    s_cl <= '1';
                    if (bit_cnt >= "001") then
                        bit_cnt <= bit_cnt - "001";
                        s_tate <= "00100101";
                    else
                        bit_cnt <= "111";
                        datacheck <= "00100110";
                        s_tate <= "00100111";
                    end if;

                --Slave ACK bit--   
                when "00100111" =>  --8'h27
                    s_cl <= '0';
                    SDA_OE <= '0';
                    SDA_OUT <= '1';
                    datacheck <= DataIn_lower;
                    s_tate <= "00101000";

                when "00101000" =>  --8'h28
                    s_cl <= '1';
                    SDA_OE <= '0';
                    a_ck <= SDA_IN;
                    if (SDA_IN = '1') then
                        datacheck <= "01010000";
                        s_tate <= "11101110";
                    else
                        datacheck <= "00101000";
                        s_tate <= "00110001";
                    end if;

                --Stop--   
                when "00110001" =>  --8'h31
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= '0';
                    datacheck <= "00110001";
                    s_tate <= "00110010";

                when "00110010" =>  --8'h32
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '0';
                    datacheck <= "00110010";
                    s_tate <= "00110011";

                when "00110011" =>  --8'h33
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';                   
                    datacheck <= "00110011";
                    s_tate <= "00000110";

                ---------Read-----------
                --Start--
                when "00000110" =>  --8'h06
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "00000110";
                    s_tate <= "00000111";                

                when "00000111" =>  --8'h07
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "00000111";
                    s_tate <= "00001000";

                when "00001000" =>  --8'h08
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '0';
                    datacheck <= "00001000";
                    s_tate <= "00001001";

               --Slave Addr+W--
                when "00001001" =>  --8'h09
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= SlaveAddress_Write (conv_integer(bit_cnt));
                    datacheck <= "00001001";
                    s_tate <= "00001010";

                when "00001010" =>  --8'h0A
                    s_cl <= '1';
                    if (bit_cnt >= "001") then   
                        bit_cnt <= bit_cnt - "001";
                        s_tate <= "00001001";
                    else
                        bit_cnt <= "111";
                        datacheck <= "00001010";
                        s_tate <= "00001011";
                    end if;

                when "00001011" =>  --8'h0B
                    s_cl <= '0';
                    SDA_OE <= '0';
                    SDA_OUT <= '1';
                    datacheck <= "00001011";
                    s_tate <= "00001100";

                --Slave ACK--   
                when "00001100" =>  --8'h0C
                    s_cl <= '1';
                    SDA_OE <= '0';
                    a_ck <= SDA_IN;
                    datacheck <= SlaveAddress_Write;
                    if (SDA_IN = '1') then
                        datacheck <= "00010000";
                        s_tate <= "11101110";
                    else
                        datacheck <= "00001100";
                        s_tate <= "00001101";
                    end if;

                --Register Access--
                when "00001101" =>  --8'h0D
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= RegAddress (conv_integer(bit_cnt));
                    datacheck <= "00001101";
                    s_tate <= "00001110";

                when "00001110" =>  --8'h0E
                    s_cl <= '1';
                    if (bit_cnt >= "001") then
                        bit_cnt <= bit_cnt - "001";
                        s_tate <= "00001101";
                    else
                        bit_cnt <= "111";
                        datacheck <= "00001110";
                        s_tate <= "00001111";
                    end if;

                when "00001111" =>  --8'h0F
                    s_cl <= '0';
                    SDA_OE <= '0';
                    SDA_OUT <= '1';
                    datacheck <= "00001111";
                    s_tate <= "00010101";

                --Slave ACK--   
                when "00010101" =>  --8'h15
                    s_cl <= '1';
                    SDA_OE <= '0';
                    a_ck <= SDA_IN;
                    datacheck <= RegAddress;
                    if (SDA_IN = '1') then
                        datacheck <= "00100000";
                        s_tate <= "11101110";
                    else
                        datacheck <= "00010101";
                        s_tate <= "00110100";
                    end if;

                --Restart--
                when "00110100"=>   --8'h34
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "00110100";
                    s_tate <= "00110110";

                when "00110110" =>  --8'h36
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "00110110";
                    s_tate <= "00110111";

                when "00110111" =>  --8'h37
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '0';
                    datacheck <= "00110111";
                    s_tate <= "00111001";

                --Slave Address + Read--   
                when "00111001" =>   --8'h39
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= SlaveAddress_Read (conv_integer(bit_cnt));
                    datacheck <= "00111001";
                    s_tate <= "00111010";

                when "00111010" =>  --8'h3A
                     s_cl <= '1';
                     if (bit_cnt >= "001") then   
                         bit_cnt <= bit_cnt - "001";
                         s_tate <= "00111001";
                     else
                         bit_cnt <= "111";
                         datacheck <= "00111010";
                         s_tate <= "00111011";
                     end if;

                 --Slave ACK Bit--                       
                 when "00111011" =>  --8'h3B
                       s_cl <= '0';
                       SDA_OE <= '0';
                       SDA_OUT <= '1';
                       datacheck <= "00111011";
                       s_tate <= "00111100";

                 when "00111100" =>  --8'h3C
                       s_cl <= '1';
                       SDA_OE <= '0';
                       a_ck <= SDA_IN;
                       datacheck <= SlaveAddress_Read;
                       if (SDA_IN = '1') then
                           datacheck <= "01100000";
                           s_tate <= "11101110";
                       else
                           datacheck <= "00111100";
                           s_tate <= "01000001";
                       end if;

                when "01000001" =>  --8'h41
                    s_cl <= '0';
                    SDA_OE <= '0';
                    datacheck <= "01000001";
                    s_tate <= "01000010";

                --Read MSB Data (Bit 15 to 8)   
                when "01000010" =>  --8'h42
                    s_cl <= '1';
                    msb_data (conv_integer(bit_cnt)) <= SDA_IN;
                    if (bit_cnt >= "001") then
                        bit_cnt <= bit_cnt - "001";
                        s_tate <= "01000001";
                    else
                        bit_cnt <= "111";
                        datacheck <= "01000010";
                        s_tate <= "01000011";
                    end if;

                --Master ACK Bit--     
                when "01000011" =>  --8'h43
                    s_cl <= '0';
                    SDA_OE <= '1';
                    datacheck <= "01000011";
                    s_tate <= "01000100";               

                when "01000100" =>  --8'h44
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '0';
                    datacheck <= "01000100";
                    s_tate <= "01000101";

               --Read LSB Data (Bit 7 to 0)       
               when "01000101" =>   --8'h45
                    s_cl <= '0';
                    SDA_OE <= '0';
                    datacheck <= "01000101";
                    s_tate <= "01000110";               

               when "01000110" =>   --8'h46
                    s_cl <= '1';
                    lsb_data (conv_integer(bit_cnt)) <= SDA_IN;
                    if (bit_cnt >= "001") then
                        bit_cnt <= bit_cnt - "001";
                        s_tate <= "01000101";
                    else
                        bit_cnt <= "111";
                        datacheck <= "01000110";
                        s_tate <= "01000111";
                    end if;

                --Master NACK Bit--     
                when "01000111" =>  --8'h47
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "01000111";
                    s_tate <= "01001000";               

               when "01001000" =>   --8'h48
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "01001000";
                    s_tate <= "00010110";

               --Stop--     
               when "00010110" =>   --8'h16
                    s_cl <= '0';
                    SDA_OE <= '1';
                    SDA_OUT <= '0';
                    a_ck <= '1';
                    datacheck <= "00010110";
                    s_tate <= "00010111";

                when "00010111" =>  --8'h17
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '0';
                    datacheck <= "00010111";
                    s_tate <= "00011000";

                when "00011000" =>  --8'h18
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "00011000";
                    s_tate <= "00011000";                       

                --Error when Slave ACK Bit is high-   
                when "11101110" =>  --8'hEE
                    s_cl <= '1';
                    SDA_OE <= '1';
                    datacheck <= "11101110";
                    s_tate <= "11101110";

                --default condition--   
                when others =>
                    s_cl <= '1';
                    SDA_OE <= '1';
                    SDA_OUT <= '1';
                    datacheck <= "11111111";
            end case;
        end if;
end process;
------------------------------------------------------------------------------------------
DATA1 <= msb_data;
DATA2 <= lsb_data;

end behav;